@@ -23,9 +23,10 @@
 #include "solid.h"
 #include "config.h"
 #include "binary.h"
-#include "text.h"
 #include "common.h"
 #include "level.h"
+#include "array.h"
+#include "dir.h"
 
 #include "game_server.h"
 #include "game_client.h"
@@ -38,39 +39,11 @@
 
 #define DATELEN 20
 
-static FILE *demo_fp;
-
-static struct demo demos[MAXDEMO]; /* Array of scanned demos  */
-static int         count;          /* Number of scanned demos */
+static fs_file demo_fp;
 
 /*---------------------------------------------------------------------------*/
 
-void demo_dump_info(const struct demo *d)
-{
-    printf("Name:         %s\n"
-           "File:         %s\n"
-           "Time:         %d\n"
-           "Coins:        %d\n"
-           "Mode:         %d\n"
-           "State:        %d\n"
-           "Date:         %s"
-           "Player:       %s\n"
-           "Shot:         %s\n"
-           "Level:        %s\n"
-           "Time:         %d\n"
-           "Goal:         %d\n"
-           "Goal enabled: %d\n"
-           "Score:        %d\n"
-           "Balls:        %d\n"
-           "Total Time:   %d\n",
-           d->name, d->filename,
-           d->timer, d->coins, d->mode, d->status, ctime(&d->date),
-           d->player,
-           d->shot, d->file,
-           d->time, d->goal, d->goal_e, d->score, d->balls, d->times);
-}
-
-static int demo_header_read(FILE *fp, struct demo *d)
+static int demo_header_read(fs_file fp, struct demo *d)
 {
     int magic;
     int version;
@@ -92,7 +65,7 @@ static int demo_header_read(FILE *fp, struct demo *d)
         get_index(fp, &d->status);
         get_index(fp, &d->mode);
 
-        get_string(fp, d->player, MAXNAM);
+        get_string(fp, d->player, sizeof (d->player));
         get_string(fp, datestr, DATELEN);
 
         sscanf(datestr,
@@ -125,7 +98,7 @@ static int demo_header_read(FILE *fp, struct demo *d)
     return 0;
 }
 
-static void demo_header_write(FILE *fp, struct demo *d)
+static void demo_header_write(fs_file fp, struct demo *d)
 {
     int magic = MAGIC;
     int version = DEMO_VERSION;
@@ -158,96 +131,54 @@ static void demo_header_write(FILE *fp, struct demo *d)
 
 /*---------------------------------------------------------------------------*/
 
-/* Scan another file (used by demo_scan). */
-
-static void demo_scan_file(const char *filename)
+struct demo *demo_load(const char *path)
 {
-    FILE *fp;
-    struct demo *d = &demos[count];
+    fs_file fp;
+    struct demo *d;
 
-    if ((fp = fopen(config_user(filename), FMODE_RB)))
+    d = NULL;
+
+    if ((fp = fs_open(path, "r")))
     {
+        d = calloc(1, sizeof (struct demo));
+
         if (demo_header_read(fp, d))
         {
-            strncpy(d->filename, config_user(filename), MAXSTR);
-            strncpy(d->name,
-                    base_name(text_from_locale(d->filename), REPLAY_EXT),
-                    PATHMAX);
+            strncpy(d->filename, path, MAXSTR - 1);
+            strncpy(d->name, base_name(d->filename, ".nbr"), PATHMAX - 1);
             d->name[PATHMAX - 1] = '\0';
-
-            count++;
         }
-        fclose(fp);
+        else
+        {
+            free(d);
+            d = NULL;
+        }
+
+        fs_close(fp);
     }
+
+    return d;
 }
 
-#ifdef _WIN32
-
-int demo_scan(void)
+void demo_free(struct demo *d)
 {
-    WIN32_FIND_DATA d;
-    HANDLE h;
-
-    count = 0;
-
-    /* Scan the user directory for files. */
-
-    if ((h = FindFirstFile(config_user("*"), &d)) != INVALID_HANDLE_VALUE)
-    {
-        do
-            demo_scan_file(d.cFileName);
-        while (count < MAXDEMO && FindNextFile(h, &d));
-
-        FindClose(h);
-    }
-    return count;
+    free(d);
 }
 
-#else /* _WIN32 */
-#include <dirent.h>
+/*---------------------------------------------------------------------------*/
 
-int demo_scan(void)
+static const char *demo_path(const char *name)
 {
-    struct dirent *ent;
-    DIR  *dp;
-
-    count = 0;
-
-    /* Scan the user directory for files. */
-
-    if ((dp = opendir(config_user(""))))
-    {
-        while (count < MAXDEMO && (ent = readdir(dp)))
-            demo_scan_file(ent->d_name);
-
-        closedir(dp);
-    }
-    return count;
-}
-#endif /* _WIN32 */
-
-const char *demo_pick(void)
-{
-    int n = demo_scan();
-
-    return (n > 0) ? demos[(rand() >> 4) % n].filename : NULL;
-}
-
-const struct demo *demo_get(int i)
-{
-    return (0 <= i && i < count) ? &demos[i] : NULL;
+    static char path[MAXSTR];
+    sprintf(path, "Replays/%s.nbr", name);
+    return path;
 }
 
 /*---------------------------------------------------------------------------*/
 
 int demo_exists(const char *name)
 {
-    char buf[MAXSTR];
-
-    strcpy(buf, config_user(name));
-    strcat(buf, REPLAY_EXT);
-
-    return file_exists(buf);
+    return fs_exists(demo_path(name));
 }
 
 #define MAXSTRLEN(a) (sizeof ((a)) - 1)
@@ -351,16 +282,15 @@ int demo_play_init(const char *name, const struct level *level,
 
     memset(&demo, 0, sizeof (demo));
 
-    strncpy(demo.filename, config_user(name), MAXSTR);
-    strcat(demo.filename, REPLAY_EXT);
+    strncpy(demo.filename, demo_path(name), sizeof (demo.filename) - 1);
 
     demo.mode = mode;
     demo.date = time(NULL);
 
-    config_get_s(CONFIG_PLAYER, demo.player, MAXNAM);
+    strncpy(demo.player, config_get_s(CONFIG_PLAYER), sizeof (demo.player) - 1);
 
-    strncpy(demo.shot, level->shot, PATHMAX);
-    strncpy(demo.file, level->file, PATHMAX);
+    strncpy(demo.shot, level->shot, PATHMAX - 1);
+    strncpy(demo.file, level->file, PATHMAX - 1);
 
     demo.time   = t;
     demo.goal   = g;
@@ -369,7 +299,7 @@ int demo_play_init(const char *name, const struct level *level,
     demo.balls  = b;
     demo.times  = tt;
 
-    if ((demo_fp = fopen(demo.filename, FMODE_WB)))
+    if ((demo_fp = fs_open(demo.filename, "w")))
     {
         demo_header_write(demo_fp, &demo);
         audio_music_fade_to(2.0f, level->song);
@@ -394,15 +324,15 @@ void demo_play_stat(int status, int coins, int timer)
 {
     if (demo_fp)
     {
-        long pos = ftell(demo_fp);
+        long pos = fs_tell(demo_fp);
 
-        fseek(demo_fp, 8, SEEK_SET);
+        fs_seek(demo_fp, 8, SEEK_SET);
 
         put_index(demo_fp, &timer);
         put_index(demo_fp, &coins);
         put_index(demo_fp, &status);
 
-        fseek(demo_fp, pos, SEEK_SET);
+        fs_seek(demo_fp, pos, SEEK_SET);
     }
 }
 
@@ -410,7 +340,7 @@ void demo_play_stop(void)
 {
     if (demo_fp)
     {
-        fclose(demo_fp);
+        fs_close(demo_fp);
         demo_fp = NULL;
     }
 }
@@ -429,18 +359,16 @@ void demo_rename(const char *name)
         demo_exists(USER_REPLAY_FILE) &&
         strcmp(name, USER_REPLAY_FILE) != 0)
     {
-        strcpy(src, config_user(USER_REPLAY_FILE));
-        strcat(src, REPLAY_EXT);
+        strncpy(src, demo_path(USER_REPLAY_FILE), sizeof (src) - 1);
+        strncpy(dst, demo_path(name),             sizeof (dst) - 1);
 
-        strcpy(dst, config_user(name));
-        strcat(dst, REPLAY_EXT);
-
-        file_rename(src, dst);
+        fs_rename(src, dst);
     }
 }
 
 void demo_rename_player(const char *name, const char *player)
 {
+#if 0
     char filename[MAXSTR];
     FILE *old_fp, *new_fp;
     struct demo d;
@@ -478,7 +406,7 @@ void demo_rename_player(const char *name, const char *player)
 
                 demo_header_write(new_fp, &d);
 
-                /* 
+                /*
                  * Restore the last three fields not written by the above call.
                  */
 
@@ -507,6 +435,7 @@ void demo_rename_player(const char *name, const char *player)
         }
         fclose(old_fp);
     }
+#endif
 }
 
 /*---------------------------------------------------------------------------*/
@@ -521,14 +450,14 @@ const struct demo *curr_demo_replay(void)
 
 int demo_replay_init(const char *name, int *g, int *m, int *b, int *s, int *tt)
 {
-    demo_fp = fopen(name, FMODE_RB);
+    demo_fp = fs_open(name, "r");
 
     if (demo_fp && demo_header_read(demo_fp, &demo_replay))
     {
-        strncpy(demo_replay.filename, name, MAXSTR);
+        strncpy(demo_replay.filename, name, MAXSTR - 1);
         strncpy(demo_replay.name,
-                base_name(text_from_locale(demo_replay.filename), REPLAY_EXT),
-                PATHMAX);
+                base_name(demo_replay.filename, ".nbr"),
+                PATHMAX - 1);
 
         if (level_load(demo_replay.file, &demo_level_replay))
         {
@@ -591,7 +520,7 @@ int demo_replay_step(float dt)
                 break;
         }
 
-        if (!feof(demo_fp))
+        if (!fs_eof(demo_fp))
         {
             game_client_step(NULL);
             return 1;
@@ -604,20 +533,15 @@ void demo_replay_stop(int d)
 {
     if (demo_fp)
     {
-        fclose(demo_fp);
+        fs_close(demo_fp);
         demo_fp = NULL;
 
-        if (d) remove(demo_replay.filename);
+        if (d) fs_remove(demo_replay.filename);
     }
 }
 
-void demo_replay_dump_info(void)
-{
-    demo_dump_info(&demo_replay);
-}
-
 /*---------------------------------------------------------------------------*/
 
-FILE *demo_file(void) { return demo_fp; }
+fs_file demo_file(void) { return demo_fp; }
 
 /*---------------------------------------------------------------------------*/
