@@ -52,11 +52,7 @@ static float timer      = 0.f;          /* Clock time                        */
 
 static int status = GAME_NONE;          /* Outcome of the game               */
 
-static float game_rx;                   /* Floor rotation about X axis       */
-static float game_rz;                   /* Floor rotation about Z axis       */
-
-static float view_a;                    /* Ideal view rotation about Y axis  */
-static float view_fov;                  /* Field of view                     */
+static struct game_tilt tilt;           /* Floor rotation                    */
 
 static float view_c[3];                 /* Current view center               */
 static float view_p[3];                 /* Current view position             */
@@ -89,6 +85,14 @@ static void game_run_cmd(const union cmd *cmd)
     static const float gup[] = { 0.0f, +9.8f, 0.0f };
     static const float gdn[] = { 0.0f, -9.8f, 0.0f };
 
+    /*
+     * Neverball <= 1.5.1 does not send explicit tilt axes, rotation
+     * happens directly around view vectors.  So for compatibility if
+     * at the time of receiving tilt angles we have not yet received
+     * the tilt axes, we use the view vectors.
+     */
+    static int got_tilt_axes;
+
     float f[3];
 
     if (client_state)
@@ -102,6 +106,9 @@ static void game_run_cmd(const union cmd *cmd)
         switch (cmd->type)
         {
         case CMD_END_OF_UPDATE:
+
+            got_tilt_axes = 0;
+
             if (first_update)
             {
                 first_update = 0;
@@ -111,9 +118,9 @@ static void game_run_cmd(const union cmd *cmd)
             /* Compute gravity for particle effects. */
 
             if (status == GAME_GOAL)
-                game_comp_grav(f, gup, view_a, game_rx, game_rz);
+                game_tilt_grav(f, gup, &tilt);
             else
-                game_comp_grav(f, gdn, view_a, game_rx, game_rz);
+                game_tilt_grav(f, gdn, &tilt);
 
             /* Step particle, goal and jump effects. */
 
@@ -181,9 +188,12 @@ static void game_run_cmd(const union cmd *cmd)
 
             break;
 
-        case CMD_ROTATE:
-            game_rx = cmd->rotate.x;
-            game_rz = cmd->rotate.z;
+        case CMD_TILT_ANGLES:
+            if (!got_tilt_axes)
+                game_tilt_axes(&tilt, view_e);
+
+            tilt.rx = cmd->tiltangles.x;
+            tilt.rz = cmd->tiltangles.z;
             break;
 
         case CMD_SOUND:
@@ -319,8 +329,6 @@ static void game_run_cmd(const union cmd *cmd)
 
             v_crs(view_e[2], view_e[0], view_e[1]);
 
-            view_a = V_DEG(fatan2f(view_e[2][0], view_e[2][2]));
-
             break;
 
         case CMD_CURRENT_BALL:
@@ -368,6 +376,12 @@ static void game_run_cmd(const union cmd *cmd)
             game_compat_map = version.x == cmd->map.version.x;
             break;
 
+        case CMD_TILT_AXES:
+            got_tilt_axes = 1;
+            v_cpy(tilt.x, cmd->tiltaxes.x);
+            v_cpy(tilt.z, cmd->tiltaxes.z);
+            break;
+
         case CMD_NONE:
         case CMD_MAX:
             break;
@@ -375,7 +389,7 @@ static void game_run_cmd(const union cmd *cmd)
     }
 }
 
-void game_client_step(FILE *demo_fp)
+void game_client_step(fs_file demo_fp)
 {
     union cmd *cmdp;
 
@@ -408,7 +422,7 @@ int  game_client_init(const char *file_name)
     if (client_state)
         game_client_free();
 
-    if (!sol_load_gl(&file, config_data(file_name),
+    if (!sol_load_gl(&file, file_name,
                      config_get_d(CONFIG_TEXTURES),
                      config_get_d(CONFIG_SHADOW)))
         return (client_state = 0);
@@ -417,8 +431,7 @@ int  game_client_init(const char *file_name)
 
     client_state = 1;
 
-    game_rx = 0.0f;
-    game_rz = 0.0f;
+    game_tilt_init(&tilt);
 
     /* Initialize jump and goal states. */
 
@@ -460,13 +473,11 @@ int  game_client_init(const char *file_name)
 
     part_reset(GOAL_HEIGHT, JUMP_HEIGHT);
 
-    view_fov = (float) config_get_d(CONFIG_VIEW_FOV);
-
     ups          = 0;
     first_update = 1;
 
     back_init(grad_name, config_get_d(CONFIG_GEOMETRY));
-    sol_load_gl(&back, config_data(back_name),
+    sol_load_gl(&back, back_name,
                 config_get_d(CONFIG_TEXTURES), 0);
 
     return client_state;
@@ -715,8 +726,8 @@ static void game_draw_tilt(int d)
     /* Rotate the environment about the position of the ball. */
 
     glTranslatef(+ball_p[0], +ball_p[1] * d, +ball_p[2]);
-    glRotatef(-game_rz * d, view_e[2][0], view_e[2][1], view_e[2][2]);
-    glRotatef(-game_rx * d, view_e[0][0], view_e[0][1], view_e[0][2]);
+    glRotatef(-tilt.rz * d, tilt.z[0], tilt.z[1], tilt.z[2]);
+    glRotatef(-tilt.rx * d, tilt.x[0], tilt.x[1], tilt.x[2]);
     glTranslatef(-ball_p[0], -ball_p[1] * d, -ball_p[2]);
 }
 
@@ -761,12 +772,15 @@ static void game_draw_light(void)
 
 static void game_draw_back(int pose, int d, float t)
 {
+    if (pose == 2)
+        return;
+
     glPushMatrix();
     {
         if (d < 0)
         {
-            glRotatef(game_rz * 2, view_e[2][0], view_e[2][1], view_e[2][2]);
-            glRotatef(game_rx * 2, view_e[0][0], view_e[0][1], view_e[0][2]);
+            glRotatef(tilt.rz * 2, tilt.z[0], tilt.z[1], tilt.z[2]);
+            glRotatef(tilt.rx * 2, tilt.x[0], tilt.x[1], tilt.x[2]);
         }
 
         glTranslatef(view_p[0], view_p[1] * d, view_p[2]);
@@ -800,7 +814,7 @@ static void game_clip_refl(int d)
 
 static void game_clip_ball(int d, const float *p)
 {
-    GLdouble r, c[3], pz[4], nz[4];
+    GLdouble r, c[3], pz[4], nz[4], ny[4];
 
     /* Compute the plane giving the front of the ball, as seen from view_p. */
 
@@ -828,13 +842,24 @@ static void game_clip_ball(int d, const float *p)
     nz[2] = -pz[2];
     nz[3] = -pz[3];
 
+    /* Compute the plane giving the bottom of the ball. */
+
+    ny[0] =  0.0;
+    ny[1] = -1.0;
+    ny[2] =  0.0;
+    ny[3] = -(ny[0] * c[0] +
+              ny[1] * c[1] +
+              ny[2] * c[2]);
+
     /* Reflect these planes as necessary, and store them in the GL state. */
 
     pz[1] *= d;
     nz[1] *= d;
+    ny[1] *= d;
 
     glClipPlane(GL_CLIP_PLANE1, nz);
     glClipPlane(GL_CLIP_PLANE2, pz);
+    glClipPlane(GL_CLIP_PLANE3, ny);
 }
 
 static void game_draw_fore(int pose, const float *M, int d, float t)
@@ -856,10 +881,13 @@ static void game_draw_fore(int pose, const float *M, int d, float t)
         if (d < 0)
             glEnable(GL_CLIP_PLANE0);
 
-        if (pose)
-            sol_draw(&file, 0, 1);
-        else
+        switch (pose)
         {
+        case 1:
+            sol_draw(&file, 0, 1);
+            break;
+
+        case 0:
             /* Draw the coins. */
 
             game_draw_items(&file, t);
@@ -868,6 +896,10 @@ static void game_draw_fore(int pose, const float *M, int d, float t)
 
             sol_draw(&file, 0, 1);
 
+            /* Fall through. */
+
+        case 2:
+
             /* Draw the ball shadow. */
 
             if (d > 0 && config_get_d(CONFIG_SHADOW))
@@ -880,6 +912,8 @@ static void game_draw_fore(int pose, const float *M, int d, float t)
             /* Draw the ball. */
 
             game_draw_balls(&file, M, t);
+
+            break;
         }
 
         /* Draw the particles and light columns. */
@@ -915,7 +949,7 @@ static void game_draw_fore(int pose, const float *M, int d, float t)
 
 void game_draw(int pose, float t)
 {
-    float fov = view_fov;
+    float fov = (float) config_get_d(CONFIG_VIEW_FOV);
 
     if (jump_b) fov *= 2.f * fabsf(jump_dt - 0.5);
 
@@ -937,7 +971,7 @@ void game_draw(int pose, float t)
 
             m_xps(M, T);
 
-            /* Apply current the view. */
+            /* Apply the current view. */
 
             v_sub(v, view_c, view_p);
 
@@ -983,8 +1017,55 @@ void game_draw(int pose, float t)
             /* Draw the scene normally. */
 
             game_draw_light();
-            game_refl_all();
-            game_draw_back(pose,    +1, t);
+
+            if (reflective)
+            {
+                if (config_get_d(CONFIG_REFLECTION))
+                {
+                    /* Draw background while preserving reflections. */
+
+                    glEnable(GL_STENCIL_TEST);
+                    {
+                        glStencilFunc(GL_NOTEQUAL, 1, 0xFFFFFFFF);
+                        game_draw_back(pose, +1, t);
+                    }
+                    glDisable(GL_STENCIL_TEST);
+
+                    /* Draw mirrors. */
+
+                    game_refl_all();
+                }
+                else
+                {
+                    /* Draw background. */
+
+                    game_draw_back(pose, +1, t);
+
+                    /*
+                     * Draw mirrors, first fully opaque with a custom
+                     * material color, then blending normally with the
+                     * opaque surfaces using their original material
+                     * properties.  (Keeps background from showing
+                     * through.)
+                     */
+
+                    glEnable(GL_COLOR_MATERIAL);
+                    {
+                        glColor4f(0.0, 0.0, 0.05, 1.0);
+                        game_refl_all();
+                        glColor4f(1.0,  1.0,  1.0,  1.0);
+                    }
+                    glDisable(GL_COLOR_MATERIAL);
+
+                    game_refl_all();
+                }
+            }
+            else
+            {
+                game_draw_back(pose, +1, t);
+                game_refl_all();
+            }
+
             game_draw_fore(pose, T, +1, t);
         }
         glPopMatrix();
