@@ -3,12 +3,6 @@
 
 /*---------------------------------------------------------------------------*/
 
-#define STR_VIEW0 _("Chase")
-#define STR_VIEW1 _("Lazy")
-#define STR_VIEW2 _("Manual")
-
-/*---------------------------------------------------------------------------*/
-
 void hud_init(void);
 void hud_free(void);
 
@@ -17,6 +11,8 @@ void hud_timer(float);
 void hud_update(int);
 
 void hud_view_pulse(int);
+void hud_view_timer(float);
+void hud_view_paint();
 
 /*---------------------------------------------------------------------------*/
 
