@@ -17,7 +17,6 @@
 #include <SDL.h>
 #include <stdio.h>
 #include <string.h>
-#include <errno.h>
 
 #include "glext.h"
 #include "config.h"
@@ -28,8 +27,9 @@
 #include "progress.h"
 #include "gui.h"
 #include "set.h"
-#include "text.h"
 #include "tilt.h"
+#include "fs.h"
+#include "common.h"
 
 #include "st_conf.h"
 #include "st_title.h"
@@ -37,7 +37,8 @@
 #include "st_level.h"
 #include "st_pause.h"
 
-#define TITLE "Neverball " VERSION
+const char TITLE[] = "Neverball " VERSION;
+const char ICON[] = "icon/neverball.png";
 
 /*---------------------------------------------------------------------------*/
 
@@ -45,8 +46,8 @@ static void shot(void)
 {
     static char filename[MAXSTR];
 
-    sprintf(filename, "screen%05d.png", config_screenshot());
-    image_snap(config_user(filename));
+    sprintf(filename, "Screenshots/screen%05d.png", config_screenshot());
+    image_snap(filename);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -248,9 +249,6 @@ static int loop(void)
 static char *data_path = NULL;
 static char *demo_path = NULL;
 
-static unsigned int display_info = 0;
-static unsigned int replay_demo  = 0;
-
 #define usage \
     L_(                                                                   \
         "Usage: %s [options ...]\n"                                       \
@@ -259,7 +257,6 @@ static unsigned int replay_demo  = 0;
         "  -v, --version             show version.\n"                     \
         "  -d, --data <dir>          use 'dir' as game data directory.\n" \
         "  -r, --replay <file>       play the replay 'file'.\n"           \
-        "  -i, --info                display info about a replay.\n"      \
     )
 
 #define argument_error(option) { \
@@ -308,12 +305,6 @@ static void parse_args(int argc, char **argv)
             continue;
         }
 
-        if (strcmp(argv[i], "-i") == 0 || strcmp(argv[i], "--info")    == 0)
-        {
-            display_info = 1;
-            continue;
-        }
-
         /* Assume a single unrecognised argument is a replay name. */
 
         if (argc == 2)
@@ -322,18 +313,6 @@ static void parse_args(int argc, char **argv)
             break;
         }
     }
-
-    /* Resolve conflicts. */
-
-    if (demo_path)
-        replay_demo = display_info ? 0 : 1;
-    else
-        if (display_info)
-        {
-            /* FIXME, I'm a required option. */
-            fputs(L_("Option '--info' requires '--replay'.\n"), stderr);
-            exit(EXIT_FAILURE);
-        }
 }
 
 #undef usage
@@ -341,30 +320,80 @@ static void parse_args(int argc, char **argv)
 
 /*---------------------------------------------------------------------------*/
 
+static int is_replay(struct dir_item *item)
+{
+    return strcmp(item->path + strlen(item->path) - 4, ".nbr") == 0;
+}
+
+static int is_score(struct dir_item *item)
+{
+    return strncmp(item->path, "neverballhs-", sizeof ("neverballhs-") - 1) == 0;
+}
+
+static void make_dirs_and_migrate(void)
+{
+    Array items;
+    int i;
+
+    const char *src;
+    char *dst;
+
+    if (fs_mkdir("Replays"))
+    {
+        if ((items = fs_dir_scan("", is_replay)))
+        {
+            for (i = 0; i < array_len(items); i++)
+            {
+                src = DIR_ITEM_GET(items, i)->path;
+                dst = concat_string("Replays/", src, NULL);
+                fs_rename(src, dst);
+                free(dst);
+            }
+
+            fs_dir_free(items);
+        }
+    }
+
+    if (fs_mkdir("Scores"))
+    {
+        if ((items = fs_dir_scan("", is_score)))
+        {
+            for (i = 0; i < array_len(items); i++)
+            {
+                src = DIR_ITEM_GET(items, i)->path;
+                dst = concat_string("Scores/",
+                                    src + sizeof ("neverballhs-") - 1,
+                                    ".txt",
+                                    NULL);
+                fs_rename(src, dst);
+                free(dst);
+            }
+
+            fs_dir_free(items);
+        }
+    }
+
+    fs_mkdir("Screenshots");
+}
+
 int main(int argc, char *argv[])
 {
     SDL_Joystick *joy = NULL;
     int t1, t0, uniform;
 
-    config_exec_path = argv[0];
+    if (!fs_init(argv[0]))
+    {
+        fprintf(stderr, "Failure to initialize virtual file system: %s\n",
+                fs_error());
+        return 1;
+    }
 
     lang_init("neverball");
 
-    text_init();
-
     parse_args(argc, argv);
 
-    if (!config_data_path(data_path, SET_FILE))
-    {
-        fputs(L_("Failure to establish game data directory\n"), stderr);
-        return 1;
-    }
-
-    if (!config_user_path(NULL))
-    {
-        fputs(L_("Failure to establish config directory\n"), stderr);
-        return 1;
-    }
+    config_paths(data_path);
+    make_dirs_and_migrate();
 
     /* Initialize SDL system and subsystems */
 
@@ -379,23 +408,9 @@ int main(int argc, char *argv[])
     config_init();
     config_load();
 
-    /* Dump replay information and exit. */
-
-    if (display_info)
-    {
-        if (!progress_replay(demo_path))
-        {
-            fprintf(stderr, L_("Replay file '%s': %s\n"), demo_path,
-                    errno ?  strerror(errno) : L_("Not a replay file"));
-            return 1;
-        }
-        demo_replay_dump_info();
-        return 0;
-    }
-
     /* Initialize the joystick. */
 
-    if (SDL_NumJoysticks() > 0)
+    if (config_get_d(CONFIG_JOYSTICK) && SDL_NumJoysticks() > 0)
     {
         joy = SDL_JoystickOpen(config_get_d(CONFIG_JOYSTICK_DEVICE));
         if (joy)
@@ -409,14 +424,15 @@ int main(int argc, char *argv[])
 
     /* Initialize the video. */
 
-    if (!video_init(TITLE, "icon/neverball.png"))
+    if (!video_init(TITLE, ICON))
         return 1;
 
     init_state(&st_null);
 
     /* Initialise demo playback. */
 
-    if (replay_demo && progress_replay(demo_path))
+    if (demo_path && fs_add_path(dir_name(demo_path)) &&
+        progress_replay(base_name(demo_path, NULL)))
     {
         demo_play_goto(1);
         goto_state(&st_demo_play);
@@ -470,7 +486,7 @@ int main(int argc, char *argv[])
 
     /* Gracefully close the game */
 
-    if (SDL_JoystickOpened(0))
+    if (joy)
         SDL_JoystickClose(joy);
 
     tilt_free();
@@ -478,8 +494,6 @@ int main(int argc, char *argv[])
 
     config_save();
 
-    text_quit();
-
     return 0;
 }
 
