@@ -15,15 +15,14 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
-#include <errno.h>
 
 #include "glext.h"
 #include "config.h"
 #include "video.h"
 #include "image.h"
-#include "text.h"
 #include "set.h"
 #include "common.h"
+#include "fs.h"
 
 #include "game_server.h"
 #include "game_client.h"
@@ -40,7 +39,8 @@ struct set
     char *desc;                /* Set description            */
     char *shot;                /* Set screen-shot            */
 
-    char user_scores[PATHMAX]; /* User high-score file       */
+    char *user_scores;         /* User high-score file       */
+    char *cheat_scores;        /* Cheat mode score file      */
 
     struct score coin_score;   /* Challenge score            */
     struct score time_score;   /* Challenge score            */
@@ -51,33 +51,34 @@ struct set
     char *level_name_v[MAXLVL]; /* List of level file names   */
 };
 
-static int set_state = 0;
+#define SET_GET(a, i) ((struct set *) array_get((a), (i)))
 
-static int set;
-static int count;
+static Array sets;
+static int   curr;
 
-static struct set   set_v[MAXSET];
 static struct level level_v[MAXLVL];
 
 /*---------------------------------------------------------------------------*/
 
-static void put_score(FILE *fp, const struct score *s)
+static void put_score(fs_file fp, const struct score *s)
 {
     int j;
 
     for (j = 0; j < NSCORE; j++)
-        fprintf(fp, "%d %d %s\n", s->timer[j], s->coins[j], s->player[j]);
+        fs_printf(fp, "%d %d %s\n", s->timer[j], s->coins[j], s->player[j]);
 }
 
 void set_store_hs(void)
 {
-    const struct set *s = &set_v[set];
-    FILE *fout;
+    const struct set *s = SET_GET(sets, curr);
+    fs_file fout;
     int i;
     const struct level *l;
     char states[MAXLVL + 1];
 
-    if ((fout = fopen(config_user(s->user_scores), "w")))
+    if ((fout = fs_open(config_cheat() ?
+                        s->cheat_scores :
+                        s->user_scores, "w")))
     {
         for (i = 0; i < s->count; i++)
         {
@@ -89,7 +90,7 @@ void set_store_hs(void)
                 states[i] = 'O';
         }
         states[s->count] = '\0';
-        fprintf(fout, "%s\n",states);
+        fs_printf(fout, "%s\n",states);
 
         put_score(fout, &s->time_score);
         put_score(fout, &s->coin_score);
@@ -103,21 +104,23 @@ void set_store_hs(void)
             put_score(fout, &l->score.most_coins);
         }
 
-        fclose(fout);
+        fs_close(fout);
     }
 }
 
-static int get_score(FILE *fp, struct score *s)
+static int get_score(fs_file fp, struct score *s)
 {
     int j;
     int res = 1;
+    char line[MAXSTR];
 
     for (j = 0; j < NSCORE && res; j++)
     {
-        res = fscanf(fp, "%d %d %s\n",
-                     &s->timer[j],
-                     &s->coins[j],
-                     s->player[j]) == 3;
+        res = (fs_gets(line, sizeof (line), fp) &&
+               sscanf(line, "%d %d %s\n",
+                      &s->timer[j],
+                      &s->coins[j],
+                      s->player[j]) == 3);
     }
     return res;
 }
@@ -125,17 +128,18 @@ static int get_score(FILE *fp, struct score *s)
 /* Get the score of the set. */
 static void set_load_hs(void)
 {
-    struct set *s = &set_v[set];
-    FILE *fin;
+    struct set *s = SET_GET(sets, curr);
+    fs_file fin;
     int i;
     int res = 0;
     struct level *l;
-    const char *fn = config_user(s->user_scores);
-    char states[MAXLVL + 1];
+    const char *fn = config_cheat() ? s->cheat_scores : s->user_scores;
+    char states[MAXLVL + sizeof ("\n")];
 
-    if ((fin = fopen(fn, "r")))
+    if ((fin = fs_open(fn, "r")))
     {
-        res = fscanf(fin, "%s\n", states) == 1 && strlen(states) == s->count;
+        res = (fs_gets(states, sizeof (states), fin) &&
+               strlen(states) - 1 == s->count);
 
         for (i = 0; i < s->count && res; i++)
         {
@@ -173,14 +177,10 @@ static void set_load_hs(void)
                 get_score(fin, &l->score.most_coins);
         }
 
-        fclose(fin);
-    }
+        fs_close(fin);
 
-    if (!res && errno != ENOENT)
-    {
-        fprintf(stderr,
-                L_("Error while loading user high-score file '%s': %s\n"),
-                fn, errno ? strerror(errno) : L_("Incorrect format"));
+        if (!res)
+            fprintf(stderr, L_("Failure to load user score file '%s'"), fn);
     }
 }
 
@@ -188,15 +188,19 @@ static void set_load_hs(void)
 
 static int set_load(struct set *s, const char *filename)
 {
-    FILE *fin;
+    fs_file fin;
     char *scores, *level_name;
 
-    fin = fopen(config_data(filename), "r");
+    /* Skip "Misc" set when not in dev mode. */
+
+    if (strcmp(filename, SET_MISC) == 0 && !config_cheat())
+        return 0;
+
+    fin = fs_open(filename, "r");
 
     if (!fin)
     {
-        fprintf(stderr, L_("Cannot load the set file '%s': %s\n"),
-                filename, strerror(errno));
+        fprintf(stderr, L_("Failure to load set file '%s'\n"), filename);
         return 0;
     }
 
@@ -225,8 +229,8 @@ static int set_load(struct set *s, const char *filename)
 
         free(scores);
 
-        strncpy(s->user_scores, "neverballhs-", PATHMAX - 1);
-        strncat(s->user_scores, s->id, PATHMAX - 1 - strlen("neverballhs-"));
+        s->user_scores  = concat_string("Scores/", s->id, ".txt",       NULL);
+        s->cheat_scores = concat_string("Scores/", s->id, "-cheat.txt", NULL);
 
         s->count = 0;
 
@@ -236,7 +240,7 @@ static int set_load(struct set *s, const char *filename)
             s->count++;
         }
 
-        fclose(fin);
+        fs_close(fin);
 
         return 1;
     }
@@ -246,107 +250,161 @@ static int set_load(struct set *s, const char *filename)
     free(s->id);
     free(s->shot);
 
-    fclose(fin);
+    fs_close(fin);
 
     return 0;
 }
 
+static void set_free(struct set *s)
+{
+    int i;
+
+    free(s->name);
+    free(s->desc);
+    free(s->id);
+    free(s->shot);
+
+    free(s->user_scores);
+    free(s->cheat_scores);
+
+    for (i = 0; i < s->count; i++)
+        free(s->level_name_v[i]);
+}
+
+/*---------------------------------------------------------------------------*/
+
+static int cmp_dir_items(const void *A, const void *B)
+{
+    const struct dir_item *a = A, *b = B;
+    return strcmp(a->path, b->path);
+}
+
+static int set_is_loaded(const char *path)
+{
+    int i;
+
+    for (i = 0; i < array_len(sets); i++)
+        if (strcmp(SET_GET(sets, i)->file, path) == 0)
+            return 1;
+
+    return 0;
+}
+
+static int is_unseen_set(struct dir_item *item)
+{
+    return (strncmp(base_name(item->path, NULL), "set-", 4) == 0 &&
+            strcmp(item->path + strlen(item->path) - 4, ".txt") == 0 &&
+            !set_is_loaded(item->path));
+}
+
 int set_init()
 {
-    FILE *fin;
+    fs_file fin;
     char *name;
 
-    if (set_state)
-        set_free();
+    Array items;
+    int i;
 
-    set   = 0;
-    count = 0;
+    if (sets)
+        set_quit();
 
-    if ((fin = fopen(config_data(SET_FILE), "r")))
+    sets = array_new(sizeof (struct set));
+    curr = 0;
+
+    /*
+     * First, load the sets listed in the set file, preserving order.
+     */
+
+    if ((fin = fs_open(SET_FILE, "r")))
     {
-        while (count < MAXSET && read_line(&name, fin))
+        while (read_line(&name, fin))
         {
-            /* Skip "Misc" set when not in dev mode. */
+            struct set *s = array_add(sets);
 
-            if (strcmp(name, SET_MISC) == 0 && !config_cheat())
-            {
-                free(name);
-                continue;
-            }
-
-            if (set_load(&set_v[count], name))
-                count++;
+            if (!set_load(s, name))
+                array_del(sets);
 
             free(name);
         }
-        fclose(fin);
-
-        set_state = 1;
+        fs_close(fin);
     }
 
-    return count;
+    /*
+     * Then, scan for any remaining set description files, and add
+     * them after the first group in alphabetic order.
+     */
+
+    if ((items = fs_dir_scan("", is_unseen_set)))
+    {
+        array_sort(items, cmp_dir_items);
+
+        for (i = 0; i < array_len(items); i++)
+        {
+            struct set *s = array_add(sets);
+
+            if (!set_load(s, DIR_ITEM_GET(items, i)->path))
+                array_del(sets);
+        }
+
+        fs_dir_free(items);
+    }
+
+    return array_len(sets);
 }
 
-void set_free(void)
+void set_quit(void)
 {
-    int i, j;
+    int i;
 
-    for (i = 0; i < count; i++)
-    {
-        free(set_v[i].name);
-        free(set_v[i].desc);
-        free(set_v[i].id);
-        free(set_v[i].shot);
+    for (i = 0; i < array_len(sets); i++)
+        set_free(array_get(sets, i));
 
-        for (j = 0; j < set_v[i].count; j++)
-            free(set_v[i].level_name_v[j]);
-    }
-
-    set_state = 0;
+    array_free(sets);
+    sets = NULL;
 }
 
 /*---------------------------------------------------------------------------*/
 
 int set_exists(int i)
 {
-    return (0 <= i && i < count);
+    return (0 <= i && i < array_len(sets));
 }
 
 const char *set_id(int i)
 {
-    return set_exists(i) ? set_v[i].id : NULL;
+    return set_exists(i) ? SET_GET(sets, i)->id : NULL;
 }
 
 const char *set_name(int i)
 {
-    return set_exists(i) ? _(set_v[i].name) : NULL;
+    return set_exists(i) ? _(SET_GET(sets, i)->name) : NULL;
 }
 
 const char *set_desc(int i)
 {
-    return set_exists(i) ? _(set_v[i].desc) : NULL;
+    return set_exists(i) ? _(SET_GET(sets, i)->desc) : NULL;
 }
 
 const char *set_shot(int i)
 {
-    return set_exists(i) ? set_v[i].shot : NULL;
+    return set_exists(i) ? SET_GET(sets, i)->shot : NULL;
 }
 
 const struct score *set_time_score(int i)
 {
-    return set_exists(i) ? &set_v[i].time_score : NULL;
+    return set_exists(i) ? &SET_GET(sets, i)->time_score : NULL;
 }
 
 const struct score *set_coin_score(int i)
 {
-    return set_exists(i) ? &set_v[i].coin_score : NULL;
+    return set_exists(i) ? &SET_GET(sets, i)->coin_score : NULL;
 }
 
 /*---------------------------------------------------------------------------*/
 
-int set_level_exists(int s, int i)
+int set_level_exists(int i, int l)
 {
-    return (i >= 0 && i < set_v[s].count);
+    return (l >= 0 && l < SET_GET(sets, i)->count);
 }
 
 static void set_load_levels(void)
@@ -365,13 +423,13 @@ static void set_load_levels(void)
         "XXI", "XXII", "XXIII", "XXIV", "XXV"
     };
 
-    for (i = 0; i < set_v[set].count; i++)
+    for (i = 0; i < SET_GET(sets, curr)->count; i++)
     {
         l = &level_v[i];
 
-        level_load(set_v[set].level_name_v[i], l);
+        level_load(SET_GET(sets, curr)->level_name_v[i], l);
 
-        l->set    = &set_v[set];
+        l->set    = SET_GET(sets, curr);
         l->number = i;
 
         if (l->is_bonus)
@@ -390,7 +448,7 @@ static void set_load_levels(void)
 
 void set_goto(int i)
 {
-    set = i;
+    curr = i;
 
     set_load_levels();
     set_load_hs();
@@ -398,28 +456,23 @@ void set_goto(int i)
 
 int curr_set(void)
 {
-    return set;
+    return curr;
 }
 
 struct level *get_level(int i)
 {
-    return (i >= 0 && i < set_v[set].count) ? &level_v[i] : NULL;
+    return (i >= 0 && i < SET_GET(sets, curr)->count) ? &level_v[i] : NULL;
 }
 
 /*---------------------------------------------------------------------------*/
 
 int set_score_update(int timer, int coins, int *score_rank, int *times_rank)
 {
-    struct set *s = &set_v[set];
-    char player[MAXSTR] = "";
+    struct set *s = SET_GET(sets, curr);
+    const char *player = config_get_s(CONFIG_PLAYER);
 
-    config_get_s(CONFIG_PLAYER, player, MAXSTR);
-
-    if (score_rank)
-        *score_rank = score_coin_insert(&s->coin_score, player, timer, coins);
-
-    if (times_rank)
-        *times_rank = score_time_insert(&s->time_score, player, timer, coins);
+    score_coin_insert(&s->coin_score, score_rank, player, timer, coins);
+    score_time_insert(&s->time_score, times_rank, player, timer, coins);
 
     if ((score_rank && *score_rank < 3) || (times_rank && *times_rank < 3))
         return 1;
@@ -429,27 +482,21 @@ int set_score_update(int timer, int coins, int *score_rank, int *times_rank)
 
 void set_rename_player(int score_rank, int times_rank, const char *player)
 {
-    struct set *s = &set_v[set];
+    struct set *s = SET_GET(sets, curr);
 
-    strncpy(s->coin_score.player[score_rank], player, MAXNAM);
-    strncpy(s->time_score.player[times_rank], player, MAXNAM);
+    strncpy(s->coin_score.player[score_rank], player, MAXNAM - 1);
+    strncpy(s->time_score.player[times_rank], player, MAXNAM - 1);
 }
 
 /*---------------------------------------------------------------------------*/
 
-void level_snap(int i)
+void level_snap(int i, const char *path)
 {
     char filename[MAXSTR];
-    char *ext;
 
     /* Convert the level name to a PNG filename. */
 
-    memset(filename, 0, MAXSTR);
-
-    ext = strrchr(level_v[i].file, '.');
-    strncpy(filename, level_v[i].file,
-            ext ? ext - level_v[i].file : strlen(level_v[i].file));
-    strcat(filename, ".png");
+    sprintf(filename, "%s/%s.png", path, base_name(level_v[i].file, ".sol"));
 
     /* Initialize the game for a snapshot. */
 
@@ -467,10 +514,9 @@ void level_snap(int i)
         game_kill_fade();
         game_client_step(NULL);
         game_draw(1, 0);
+        SDL_GL_SwapBuffers();
 
         image_snap(filename);
-
-        SDL_GL_SwapBuffers();
     }
 }
 
@@ -478,7 +524,7 @@ void set_cheat(void)
 {
     int i;
 
-    for (i = 0; i < set_v[set].count; i++)
+    for (i = 0; i < SET_GET(sets, curr)->count; i++)
     {
         level_v[i].is_locked    = 0;
         level_v[i].is_completed = 1;
