@@ -7,13 +7,12 @@
 #define SET_FILE "sets.txt"
 #define SET_MISC "set-misc.txt"
 
-#define MAXSET 16
 #define MAXLVL 25
 
 /*---------------------------------------------------------------------------*/
 
 int  set_init(void);
-void set_free(void);
+void set_quit(void);
 
 /*---------------------------------------------------------------------------*/
 
@@ -39,7 +38,7 @@ void set_store_hs(void);
 int set_level_exists(int, int);
 struct level *get_level(int);
 
-void level_snap(int);
+void level_snap(int, const char *);
 void set_cheat(void);
 
 /*---------------------------------------------------------------------------*/
