@@ -22,6 +22,7 @@
 #include "demo.h"
 #include "progress.h"
 #include "text.h"
+#include "common.h"
 
 #include "game_common.h"
 
@@ -40,11 +41,11 @@ static struct state *cancel_state;
 
 int goto_save(struct state *ok, struct state *cancel)
 {
-    char fmt[MAXSTR] = "";
     const char *name;
 
-    config_get_s(CONFIG_REPLAY_NAME, fmt, sizeof (fmt) - 1);
-    name = demo_format_name(fmt, set_id(curr_set()), level_name(curr_level()));
+    name = demo_format_name(config_get_s(CONFIG_REPLAY_NAME),
+                            set_id(curr_set()),
+                            level_name(curr_level()));
 
     strncpy(filename, name, sizeof (filename) - 1);
 
@@ -70,7 +71,7 @@ static int save_action(int i)
     switch (i)
     {
     case SAVE_SAVE:
-        n = text_to_locale(filename);
+        n = filename;
 
         if (strlen(n) == 0)
             return 1;
@@ -96,8 +97,7 @@ static int save_action(int i)
         break;
 
     default:
-        if (text_add_char(i, filename, sizeof (filename) - 1,
-                                       sizeof (filename) - 1))
+        if (!path_is_sep(i) && text_add_char(i, filename, sizeof (filename)))
             gui_set_label(file_id, filename);
     }
     return 1;
@@ -114,7 +114,7 @@ static int save_enter(void)
         gui_label(id, _("Replay Name"), GUI_MED, GUI_ALL, 0, 0);
         gui_space(id);
 
-        file_id = gui_label(id, filename, GUI_MED, GUI_ALL, gui_yel, gui_yel);
+        file_id = gui_label(id, " ", GUI_MED, GUI_ALL, gui_yel, gui_yel);
 
         gui_space(id);
         gui_keyboard(id);
@@ -128,6 +128,9 @@ static int save_enter(void)
         }
 
         gui_layout(id, 0, 0);
+
+        gui_set_trunc(file_id, TRUNC_HEAD);
+        gui_set_label(file_id, filename);
     }
 
     SDL_EnableUNICODE(1);
@@ -182,7 +185,7 @@ static int clobber_action(int i)
 
     if (i == SAVE_SAVE)
     {
-        demo_rename(text_to_locale(filename));
+        demo_rename(filename);
         return goto_state(ok_state);
     }
     return goto_state(&st_save);
@@ -191,12 +194,13 @@ static int clobber_action(int i)
 static int clobber_enter(void)
 {
     int id, jd, kd;
+    int file_id;
 
     if ((id = gui_vstack(0)))
     {
         kd = gui_label(id, _("Overwrite?"), GUI_MED, GUI_ALL, gui_red, gui_red);
 
-        gui_label(id, filename, GUI_MED, GUI_ALL, gui_yel, gui_yel);
+        file_id = gui_label(id, "MMMMMMMM", GUI_MED, GUI_ALL, gui_yel, gui_yel);
 
         if ((jd = gui_harray(id)))
         {
@@ -206,6 +210,9 @@ static int clobber_enter(void)
 
         gui_pulse(kd, 1.2f);
         gui_layout(id, 0, 0);
+
+        gui_set_trunc(file_id, TRUNC_TAIL);
+        gui_set_label(file_id, filename);
     }
 
     return id;
