@@ -13,6 +13,7 @@
  */
 
 #include <string.h>
+#include <assert.h>
 
 #include "gui.h"
 #include "vec3.h"
@@ -21,6 +22,7 @@
 #include "config.h"
 #include "st_shared.h"
 #include "cmd.h"
+#include "demo_dir.h"
 
 #include "game_common.h"
 #include "game_server.h"
@@ -52,11 +54,19 @@ static int init_title_level(void)
     return 0;
 }
 
+static const char *pick_demo(Array items)
+{
+    struct dir_item *item;
+    return (item = array_rnd(items)) ? item->path : NULL;
+}
+
 /*---------------------------------------------------------------------------*/
 
 static float real_time = 0.0f;
 static int   mode      = 0;
 
+static Array items;
+
 static int play_id = 0;
 
 #define TITLE_PLAY 1
@@ -72,16 +82,12 @@ static int title_action(int i)
 
     size_t queue_len = strlen(queue);
 
-    char player[MAXNAM];
-
     audio_play(AUD_MENU, 1.0f);
 
     switch (i)
     {
     case TITLE_PLAY:
-        config_get_s(CONFIG_PLAYER, player, MAXNAM);
-
-        if (strlen(player) == 0)
+        if (strlen(config_get_s(CONFIG_PLAYER)) == 0)
             return goto_name(&st_set, &st_title, 0);
         else
             return goto_state(&st_set);
@@ -187,6 +193,12 @@ static int title_enter(void)
 
 static void title_leave(int id)
 {
+    if (items)
+    {
+        demo_dir_free(items);
+        items = NULL;
+    }
+
     SDL_EnableUNICODE(0);
     demo_replay_stop(0);
     gui_delete(id);
@@ -220,7 +232,10 @@ static void title_timer(int id, float dt)
 
         if (real_time > 1.0f)
         {
-            if ((demo = demo_pick()))
+            if (!items)
+                items = demo_dir_scan();
+
+            if ((demo = pick_demo(items)))
             {
                 demo_replay_init(demo, NULL, NULL, NULL, NULL, NULL);
                 game_set_fly(0.0f, game_client_file());
