@@ -21,7 +21,7 @@
 
 #define GUI_MOST_COINS  GUI_MSK(8)
 #define GUI_BEST_TIMES  GUI_MSK(16)
-#define GUI_UNLOCK_GOAL GUI_MSK(32)
+#define GUI_FAST_UNLOCK GUI_MSK(32)
 
 void gui_score_set(int);
 int  gui_score_get(void);
