@@ -16,9 +16,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "common.h"
 #include "config.h"
 #include "course.h"
 #include "hole.h"
+#include "fs.h"
 
 /*---------------------------------------------------------------------------*/
 
@@ -39,30 +41,93 @@ static struct course course_v[MAXCRS];
 
 /*---------------------------------------------------------------------------*/
 
+static int course_load(struct course *course, const char *filename)
+{
+    fs_file fin;
+    int rc = 0;
+
+    memset(course, 0, sizeof (*course));
+
+    strncpy(course->holes, filename, MAXSTR - 1);
+
+    if ((fin = fs_open(filename, "r")))
+    {
+        if (fs_gets(course->shot, sizeof (course->shot), fin) &&
+            fs_gets(course->desc, sizeof (course->desc), fin))
+        {
+            strip_newline(course->shot);
+            strip_newline(course->desc);
+
+            rc = 1;
+        }
+
+        fs_close(fin);
+    }
+
+    return rc;
+}
+
+static int cmp_dir_items(const void *A, const void *B)
+{
+    const struct dir_item *a = A, *b = B;
+    return strcmp(a->path, b->path);
+}
+
+static int course_is_loaded(const char *path)
+{
+    int i;
+
+    for (i = 0; i < count; i++)
+        if (strcmp(course_v[i].holes, path) == 0)
+            return 1;
+
+    return 0;
+}
+
+static int is_unseen_course(struct dir_item *item)
+{
+    return (strncmp(base_name(item->path, NULL), "holes-", 6) == 0 &&
+            strcmp(item->path + strlen(item->path) - 4, ".txt") == 0 &&
+            !course_is_loaded(item->path));
+}
+
 void course_init()
 {
-    FILE *fin;
+    fs_file fin;
+    char *line;
+
+    Array items;
+    int i;
 
     if (course_state)
         course_free();
 
     count = 0;
 
-    if ((fin = fopen(config_data(COURSE_FILE), "r")))
+    if ((fin = fs_open(COURSE_FILE, "r")))
     {
-        while (fscanf(fin, "%s %s\n",
-                      course_v[count].holes,
-                      course_v[count].shot) == 2 &&
-                fgets(course_v[count].desc, MAXSTR, fin))
+        while (count < MAXCRS && read_line(&line, fin))
         {
-            char *q = course_v[count].desc + strlen(course_v[count].desc) - 1;
+            if (course_load(&course_v[count], line))
+                count++;
 
-            if (*q == '\n') *q = 0;
-
-            count++;
+            free(line);
         }
 
-        fclose(fin);
+        fs_close(fin);
+
+        course_state = 1;
+    }
+
+    if ((items = fs_dir_scan("", is_unseen_course)))
+    {
+        array_sort(items, cmp_dir_items);
+
+        for (i = 0; i < array_len(items) && count < MAXCRS; i++)
+            if (course_load(&course_v[count], DIR_ITEM_GET(items, i)->path))
+                count++;
+
+        fs_dir_free(items);
 
         course_state = 1;
     }
