@@ -1,4 +1,4 @@
-/*   
+/*
  * Copyright (C) 2003 Robert Kooima
  *
  * NEVERPUTT is  free software; you can redistribute  it and/or modify
@@ -42,13 +42,13 @@ void hud_init(void)
     if ((Lhud_id = gui_hstack(0)))
     {
         gui_label(Lhud_id, curr_scr(), GUI_MED, GUI_NE, color[i], gui_wht);
-        gui_label(Lhud_id, "Score",    GUI_SML, 0,      gui_wht,  gui_wht);
+        gui_label(Lhud_id, _("Score"), GUI_SML, 0,      gui_wht,  gui_wht);
         gui_layout(Lhud_id, -1, -1);
     }
     if ((Rhud_id = gui_hstack(0)))
     {
         gui_label(Rhud_id, curr_par(), GUI_MED, 0,      color[i], gui_wht);
-        gui_label(Rhud_id, "Par",      GUI_SML, GUI_NW, gui_wht,  gui_wht);
+        gui_label(Rhud_id, _("Par"),   GUI_SML, GUI_NW, gui_wht,  gui_wht);
         gui_layout(Rhud_id, +1, -1);
     }
     if ((fps_id = gui_count(0, 1000, GUI_SML, GUI_SE)))
