@@ -24,6 +24,7 @@
 #include "audio.h"
 #include "course.h"
 #include "config.h"
+#include "video.h"
 
 #include "st_all.h"
 #include "st_conf.h"
@@ -254,7 +255,7 @@ static void title_point(int id, int x, int y, int dx, int dy)
 
 static int title_click(int b, int d)
 {
-    return (d && b < 0) ? title_action(gui_token(gui_click())) : 1;
+    return d && b == SDL_BUTTON_LEFT ? title_action(gui_token(gui_click())) : 1;
 }
 
 static int title_buttn(int b, int d)
@@ -403,7 +404,7 @@ static void course_stick(int id, int a, int v)
 
 static int course_click(int b, int d)
 {
-    return (d && b < 0) ? course_action(gui_token(gui_click())) : 1;
+    return d && b == SDL_BUTTON_LEFT ? course_action(gui_token(gui_click())) : 1;
 }
 
 static int course_buttn(int b, int d)
@@ -520,7 +521,7 @@ static void party_point(int id, int x, int y, int dx, int dy)
 
 static int party_click(int b, int d)
 {
-    return (d && b < 0) ? party_action(gui_token(gui_click())) : 1;
+    return d && b == SDL_BUTTON_LEFT ? party_action(gui_token(gui_click())) : 1;
 }
 
 static int party_buttn(int b, int d)
@@ -626,7 +627,7 @@ static void pause_point(int id, int x, int y, int dx, int dy)
 
 static int pause_click(int b, int d)
 {
-    return (d && b < 0) ? pause_action(gui_token(gui_click())) : 1;
+    return d && b == SDL_BUTTON_LEFT ? pause_action(gui_token(gui_click())) : 1;
 }
 
 static int pause_keybd(int c, int d)
@@ -734,7 +735,7 @@ static void next_point(int id, int x, int y, int dx, int dy)
 
 static int next_click(int b, int d)
 {
-    return (d && b < 0) ? goto_state(&st_flyby) : 1;
+    return (d && b == SDL_BUTTON_LEFT) ? goto_state(&st_flyby) : 1;
 }
 
 static int next_keybd(int c, int d)
@@ -833,7 +834,7 @@ static void flyby_timer(int id, float dt)
 
 static int flyby_click(int b, int d)
 {
-    if (d && b < 0)
+    if (d && b == SDL_BUTTON_LEFT)
     {
         game_set_fly(0.f);
         return goto_state(&st_stroke);
@@ -866,7 +867,7 @@ static int stroke_enter(void)
     hud_init();
     game_clr_mag();
     config_set_d(CONFIG_CAMERA, 2);
-    config_set_grab(!paused);
+    video_set_grab(!paused);
 
     if (paused)
         paused = 0;
@@ -877,7 +878,7 @@ static int stroke_enter(void)
 static void stroke_leave(int id)
 {
     hud_free();
-    config_clr_grab();
+    video_clr_grab();
     config_set_d(CONFIG_CAMERA, 0);
 }
 
@@ -926,7 +927,7 @@ static void stroke_stick(int id, int a, int v)
 
 static int stroke_click(int b, int d)
 {
-    return (d && b < 0) ? goto_state(&st_roll) : 1;
+    return (d && b == SDL_BUTTON_LEFT) ? goto_state(&st_roll) : 1;
 }
 
 static int stroke_buttn(int b, int d)
@@ -1033,7 +1034,7 @@ static void goal_timer(int id, float dt)
 
 static int goal_click(int b, int d)
 {
-    if (b < 0 && d == 1)
+    if (b == SDL_BUTTON_LEFT && d == 1)
     {
         if (hole_next())
             goto_state(&st_next);
@@ -1103,7 +1104,7 @@ static void stop_timer(int id, float dt)
 
 static int stop_click(int b, int d)
 {
-    if (b < 0 && d == 1)
+    if (b == SDL_BUTTON_LEFT && d == 1)
     {
         if (hole_next())
             goto_state(&st_next);
@@ -1178,7 +1179,7 @@ static void fall_timer(int id, float dt)
 
 static int fall_click(int b, int d)
 {
-    if (b < 0 && d == 1)
+    if (b == SDL_BUTTON_LEFT && d == 1)
     {
         if (hole_next())
             goto_state(&st_next);
@@ -1235,7 +1236,7 @@ static void score_timer(int id, float dt)
 
 static int score_click(int b, int d)
 {
-    if (b < 0 && d == 1)
+    if (b == SDL_BUTTON_LEFT && d == 1)
     {
         if (hole_move())
             return goto_state(&st_next);
@@ -1288,7 +1289,7 @@ static void over_timer(int id, float dt)
 
 static int over_click(int b, int d)
 {
-    return (d && b < 0) ? goto_state(&st_title) : 1;
+    return (d && b == SDL_BUTTON_LEFT) ? goto_state(&st_title) : 1;
 }
 
 static int over_buttn(int b, int d)
