@@ -16,5 +16,10 @@ extern struct state st_stop;
 extern struct state st_fall;
 extern struct state st_score;
 extern struct state st_over;
+extern struct state st_pause;
+
+void set_joystick(SDL_Joystick *);
+
+int goto_pause(struct state *, int);
 
 #endif
