@@ -3,11 +3,12 @@
 
 /*---------------------------------------------------------------------------*/
 
-#define BACK_DIST 150.0
+#define BACK_DIST 256.0f
+#define FAR_DIST  512.0f
 
 void back_init(const char *, int);
 void back_free(void);
-void back_draw(double);
+void back_draw(float);
 
 /*---------------------------------------------------------------------------*/
 
