@@ -55,7 +55,6 @@
 
 #define USER_CONFIG_FILE    "neverballrc"
 #define USER_REPLAY_FILE    "Last"
-#define REPLAY_EXT          ".nbr"
 
 /*---------------------------------------------------------------------------*/
 
@@ -89,20 +88,12 @@
 #define PATHMAX 64
 #define MAXNAM 9
 
-#define GUI_FACE                    _("ttf/DejaVuSans-Bold.ttf")
-
 #define UPS 90
 #define DT  (1.0f / (float) UPS)
 
 /*---------------------------------------------------------------------------*/
 
-const char *config_data(const char *);
-const char *config_user(const char *);
-
-extern const char *config_exec_path;
-
-int  config_data_path(const char *, const char *);
-int  config_user_path(const char *);
+void config_paths(const char *);
 
 /*---------------------------------------------------------------------------*/
 
