@@ -20,80 +20,57 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <errno.h>
 #include <time.h>
 #include <ctype.h>
+#include <stdarg.h>
+#include <assert.h>
 
 #include "common.h"
+#include "fs.h"
 
 #define MAXSTR 256
 
 /*---------------------------------------------------------------------------*/
 
-int read_line(char **dst, FILE *fin)
+int read_line(char **dst, fs_file fin)
 {
-    char buffer[MAXSTR] = "";
-    int  buffer_size    = 0;
+    char buff[MAXSTR];
 
-    char *store      = NULL;
-    char *store_new  = NULL;
-    int   store_size = 0;
+    char *line, *new;
+    size_t len0, len1;
 
-    int seen_newline = 0;
+    line = NULL;
 
-    while (!seen_newline)
+    while (fs_gets(buff, sizeof (buff), fin))
     {
-        if (fgets(buffer, sizeof (buffer), fin) == NULL)
+        /* Append to data read so far. */
+
+        if (line)
         {
-            if (store_size > 0)
-                break;
-            else
-            {
-                *dst = NULL;
-                return 0;
-            }
-        }
-
-        buffer_size = strlen(buffer) + 1;
-
-        /* Erase trailing newline. */
-
-        if (buffer[buffer_size - 2] == '\n')
-        {
-            seen_newline = 1;
-            buffer[buffer_size - 2] = '\0';
-            buffer_size--;
-        }
-
-        /* Allocate or reallocate space for the buffer. */
-
-        if ((store_new = (char *) realloc(store, store_size + buffer_size)))
-        {
-            /* Avoid passing garbage to string functions. */
-
-            if (store == NULL)
-                store_new[0] = '\0';
-
-            store       = store_new;
-            store_size += buffer_size;
-
-            store_new = NULL;
+            new  = concat_string(line, buff, NULL);
+            free(line);
+            line = new;
         }
         else
         {
-            fprintf(stderr, "Failed to allocate memory.\n");
-
-            free(store);
-            *dst = NULL;
-            return 0;
+            line = strdup(buff);
         }
 
-        strncat(store, buffer, buffer_size);
+        /* Strip newline, if any. */
+
+        len0 = strlen(line);
+        strip_newline(line);
+        len1 = strlen(line);
+
+        if (len1 != len0)
+        {
+            /* We hit a newline, clean up and break. */
+            line = realloc(line, len1 + 1);
+            break;
+        }
     }
 
-    *dst = store;
-
-    return 1;
+    return (*dst = line) ? 1 : 0;
 }
 
 char *strip_newline(char *str)
@@ -116,6 +93,52 @@ char *dupe_string(const char *src)
     return dst;
 }
 
+char *concat_string(const char *first, ...)
+{
+    char *full;
+
+    if ((full = strdup(first)))
+    {
+        const char *part;
+        va_list ap;
+
+        va_start(ap, first);
+
+        while ((part = va_arg(ap, const char *)))
+        {
+            char *new;
+
+            if ((new = realloc(full, strlen(full) + strlen(part) + 1)))
+            {
+                full = new;
+                strcat(full, part);
+            }
+            else
+            {
+                free(full);
+                full = NULL;
+                break;
+            }
+        }
+
+        va_end(ap);
+    }
+
+    return full;
+}
+
+char *trunc_string(const char *src, char *dst, int len)
+{
+    static const char ell[] = "...";
+
+    assert(len > sizeof (ell));
+
+    if (dst[len - 1] = '\0', strncpy(dst, src, len), dst[len - 1] != '\0')
+        strcpy(dst + len - sizeof (ell), ell);
+
+    return dst;
+}
+
 time_t make_time_from_utc(struct tm *tm)
 {
     struct tm local, *utc;
@@ -232,7 +255,7 @@ char *base_name(const char *name, const char *suffix)
 
     base = path_last_sep(name);
 
-    strncpy(buf, base ? base + 1 : name, sizeof (buf));
+    strncpy(buf, base ? base + 1 : name, sizeof (buf) - 1);
 
     /* Remove the suffix. */
 
@@ -291,3 +314,10 @@ char *path_resolve(const char *ref, const char *rel)
 }
 
 /*---------------------------------------------------------------------------*/
+
+int rand_between(int low, int high)
+{
+    return low + rand() / (RAND_MAX / (high - low + 1) + 1);
+}
+
+/*---------------------------------------------------------------------------*/
