@@ -16,9 +16,12 @@
 #define GUI_H
 
 #include "glext.h"
+#include "lang.h"
 
 /*---------------------------------------------------------------------------*/
 
+#define GUI_FACE N_("ttf/DejaVuSans-Bold.ttf")
+
 #define GUI_SML  0
 #define GUI_MED  1
 #define GUI_LRG  2
@@ -49,12 +52,20 @@ void gui_free(void);
 
 /*---------------------------------------------------------------------------*/
 
+enum trunc
+{
+    TRUNC_NONE,
+    TRUNC_HEAD,
+    TRUNC_TAIL
+};
+
 void gui_set_label(int, const char *);
 void gui_set_image(int, const char *);
 void gui_set_multi(int, const char *);
 void gui_set_count(int, int);
 void gui_set_clock(int, int);
 void gui_set_color(int, const float *, const float *);
+void gui_set_trunc(int, enum trunc);
 
 /*---------------------------------------------------------------------------*/
 
