@@ -1,4 +1,4 @@
-/*   
+/*
  * Copyright (C) 2003 Robert Kooima
  *
  * NEVERBALL is  free software; you can redistribute  it and/or modify
@@ -15,10 +15,8 @@
 #ifndef SOL_H
 #define SOL_H
 
-#include <SDL.h>
 #include "glext.h"
-
-#define PATHMAX 64
+#include "base_config.h"
 
 /*
  * Some might  be taken  aback at  the terseness of  the names  of the
@@ -33,7 +31,7 @@
  * the variable.
  *
  * The Xs are as documented by struct s_file:
- * 
+ *
  *     f  File          (struct s_file)
  *     m  Material      (struct s_mtrl)
  *     v  Vertex        (struct s_vert)
@@ -45,13 +43,15 @@
  *     n  Node          (struct s_node)
  *     p  Path          (struct s_path)
  *     b  Body          (struct s_body)
- *     c  Coin          (struct s_coin)
+ *     h  Item          (struct s_item)
  *     z  Goal          (struct s_goal)
  *     j  Jump          (struct s_jump)
  *     x  Switch        (struct s_swch)
  *     r  Billboard     (struct s_bill)
  *     u  User          (struct s_ball)
- *     i  Index         (short)
+ *     w  Viewpoint     (struct s_view)
+ *     d  Dictionary    (struct s_dict)
+ *     i  Index         (int)
  *     a  Text          (char)
  *
  * The Ys are as follows:
@@ -68,11 +68,11 @@
  * lumps.  "ei" and "ej" are  edge indices into some "ev" edge vector.
  * An edge is  defined by two vertices, so  an edge structure consists
  * of "vi" and "vj".  And so on.
- * 
+ *
  * Those members that do not conform to this convention are explicitly
  * documented with a comment.
  *
- * These prefixes are still available: d h k o q w y.
+ * These prefixes are still available: c k o q y.
  */
 
 /*---------------------------------------------------------------------------*/
@@ -85,29 +85,39 @@
 #define M_ENVIRONMENT  8
 #define M_ADDITIVE    16
 #define M_CLAMPED     32
-#define M_SHADOWED (M_OPAQUE | M_REFLECTIVE | M_ENVIRONMENT)
+#define M_DECAL       64
+#define M_TWO_SIDED  128
 
 /* Billboard types. */
 
 #define B_EDGE     1
 #define B_FLAT     2
 #define B_ADDITIVE 4
+#define B_NOFACE   8
 
 /* Lump flags. */
 
-#define L_DETAIL       1
+#define L_DETAIL   1
+
+/* Item types. */
+
+#define ITEM_NONE       0
+#define ITEM_COIN       1
+#define ITEM_GROW       2
+#define ITEM_SHRINK     3
 
 /*---------------------------------------------------------------------------*/
 
 struct s_mtrl
 {
-    float a[4];                                /* ambient color              */
     float d[4];                                /* diffuse color              */
+    float a[4];                                /* ambient color              */
     float s[4];                                /* specular color             */
     float e[4];                                /* emission color             */
     float h[1];                                /* specular exponent          */
+    float angle;
 
-    short fl;                                  /* material flags             */
+    int fl;                                    /* material flags             */
 
     GLuint o;                                  /* OpenGL texture object      */
     char   f[PATHMAX];                         /* texture file name          */
@@ -120,8 +130,8 @@ struct s_vert
 
 struct s_edge
 {
-    short vi;
-    short vj;
+    int vi;
+    int vj;
 };
 
 struct s_side
@@ -137,28 +147,28 @@ struct s_texc
 
 struct s_geom
 {
-    short mi;
-    short ti, si, vi;
-    short tj, sj, vj;
-    short tk, sk, vk;
+    int mi;
+    int ti, si, vi;
+    int tj, sj, vj;
+    int tk, sk, vk;
 };
 
 struct s_lump
 {
-    short fl;                                  /* lump flags                 */
-    short v0, vc;
-    short e0, ec;
-    short g0, gc;
-    short s0, sc;
+    int fl;                                    /* lump flags                 */
+    int v0, vc;
+    int e0, ec;
+    int g0, gc;
+    int s0, sc;
 };
 
 struct s_node
 {
-    short si;
-    short ni;
-    short nj;
-    short l0;
-    short lc;
+    int si;
+    int ni;
+    int nj;
+    int l0;
+    int lc;
 };
 
 struct s_path
@@ -166,8 +176,9 @@ struct s_path
     float p[3];                                /* starting position          */
     float t;                                   /* travel time                */
 
-    short pi;
-    short f;                                   /* enable flag                */
+    int pi;
+    int f;                                     /* enable flag                */
+    int s;                                     /* smooth flag                */
 };
 
 struct s_body
@@ -179,18 +190,19 @@ struct s_body
     GLuint rl;                                 /* reflective geometry list   */
     GLuint sl;                                 /* shadowed geometry list     */
 
-    short pi;
-    short ni;
-    short l0;
-    short lc;
-    short g0;
-    short gc;
+    int pi;
+    int ni;
+    int l0;
+    int lc;
+    int g0;
+    int gc;
 };
 
-struct s_coin
+struct s_item
 {
     float p[3];                                /* position                   */
-    short n;                                   /* value                      */
+    int   t;                                   /* type                       */
+    int   n;                                   /* value                      */
 };
 
 struct s_goal
@@ -203,18 +215,20 @@ struct s_swch
 {
     float p[3];                                /* position                   */
     float r;                                   /* radius                     */
-    short pi;
+    int  pi;                                   /* the linked path            */
 
     float t0;                                  /* default timer              */
     float t;                                   /* current timer              */
-    short f0;                                  /* default state              */
-    short f;                                   /* current state              */
+    int   f0;                                  /* default state              */
+    int   f;                                   /* current state              */
+    int   i;                                   /* is invisible?              */
+    int   e;                                   /* is a ball inside it?       */
 };
 
 struct s_bill
 {
-    short fl;
-    short mi;
+    int  fl;
+    int  mi;
     float t;                                   /* repeat time interval       */
     float d;                                   /* distance                   */
 
@@ -224,6 +238,8 @@ struct s_bill
     float rx[3];                               /* X rotation coefficients    */
     float ry[3];                               /* Y rotation coefficients    */
     float rz[3];                               /* Z rotation coefficients    */
+
+    float p[3];
 };
 
 struct s_jump
@@ -239,6 +255,8 @@ struct s_ball
     float p[3];                                /* position vector            */
     float v[3];                                /* velocity vector            */
     float w[3];                                /* angular velocity vector    */
+    float E[3][3];                             /* basis of pendulum          */
+    float W[3];                                /* angular pendulum velocity  */
     float r;                                   /* radius                     */
 };
 
@@ -248,28 +266,36 @@ struct s_view
     float q[3];
 };
 
+struct s_dict
+{
+    int ai;
+    int aj;
+};
+
 struct s_file
 {
-    short mc;
-    short vc;
-    short ec;
-    short sc;
-    short tc;
-    short gc;
-    short lc;
-    short nc;
-    short pc;
-    short bc;
-    short cc;
-    short zc;
-    short jc;
-    short xc;
-    short rc;
-    short uc;
-    short wc;
-    short ac;
-    short ic;
+    int ac;
+    int mc;
+    int vc;
+    int ec;
+    int sc;
+    int tc;
+    int gc;
+    int lc;
+    int nc;
+    int pc;
+    int bc;
+    int hc;
+    int zc;
+    int jc;
+    int xc;
+    int rc;
+    int uc;
+    int wc;
+    int dc;
+    int ic;
 
+    char          *av;
     struct s_mtrl *mv;
     struct s_vert *vv;
     struct s_edge *ev;
@@ -280,40 +306,24 @@ struct s_file
     struct s_node *nv;
     struct s_path *pv;
     struct s_body *bv;
-    struct s_coin *cv;
+    struct s_item *hv;
     struct s_goal *zv;
     struct s_jump *jv;
     struct s_swch *xv;
     struct s_bill *rv;
     struct s_ball *uv;
     struct s_view *wv;
-    short         *iv;
-    char          *av;
+    struct s_dict *dv;
+    int           *iv;
 };
 
 /*---------------------------------------------------------------------------*/
 
-int   sol_load(struct s_file *, const char *, int, int);
+int   sol_load_only_file(struct s_file *, const char *);
+int   sol_load_only_head(struct s_file *, const char *);
 int   sol_stor(struct s_file *, const char *);
 void  sol_free(struct s_file *);
 
-void  sol_back(const struct s_file *, float, float, float);
-void  sol_refl(const struct s_file *);
-void  sol_draw(const struct s_file *);
-void  sol_shad(const struct s_file *);
-
-float sol_step(struct s_file *, const float *, float, short, int *);
-
-int   sol_coin_test(struct s_file *, float *, float);
-int   sol_goal_test(struct s_file *, float *, short);
-int   sol_jump_test(struct s_file *, float *, short);
-int   sol_swch_test(struct s_file *, int, short);
-
-/*---------------------------------------------------------------------------*/
-
-void put_file_state(FILE *, struct s_file *);
-void get_file_state(FILE *, struct s_file *);
-
 /*---------------------------------------------------------------------------*/
 
 #endif
