@@ -1,4 +1,4 @@
-/*   
+/*
  * Copyright (C) 2003 Robert Kooima
  *
  * NEVERBALL is  free software; you can redistribute  it and/or modify
@@ -62,10 +62,14 @@ void m_xps(float *M, const float *N)
     M[C] = N[3]; M[D] = N[7]; M[E] = N[B]; M[F] = N[F];
 }
 
-int  m_inv(float *I, const float *M)
+int  m_inv(float *I, const float *N)
 {
-    float T[16];
-    float d;
+    double T[16], M[16];
+    double d;
+    int i;
+
+    for (i = 0; i < 16; i++)
+        M[i] = N[i];
 
     T[0] = +(M[5] * (M[A] * M[F] - M[B] * M[E]) -
              M[9] * (M[6] * M[F] - M[7] * M[E]) +
@@ -195,9 +199,9 @@ void m_rot(float *M, const float *v, float a)
 
     v_nrm(u, v);
 
-    U[0] = u[0] * u[0]; U[4] = u[0] * u[1]; U[8] = u[0] * u[2]; 
-    U[1] = u[1] * u[0]; U[5] = u[1] * u[1]; U[9] = u[1] * u[2]; 
-    U[2] = u[2] * u[0]; U[6] = u[2] * u[1]; U[A] = u[2] * u[2]; 
+    U[0] = u[0] * u[0]; U[4] = u[0] * u[1]; U[8] = u[0] * u[2];
+    U[1] = u[1] * u[0]; U[5] = u[1] * u[1]; U[9] = u[1] * u[2];
+    U[2] = u[2] * u[0]; U[6] = u[2] * u[1]; U[A] = u[2] * u[2];
 
     S[0] =   0.f; S[4] = -u[2]; S[8] =  u[1];
     S[1] =  u[2]; S[5] =   0.f; S[9] = -u[0];
@@ -262,3 +266,24 @@ void m_vxfm(float *v, const float *M, const float *w)
     v[2] = (w[0] * M[2] + w[1] * M[6] + w[2] * M[A]);
 }
 
+/*---------------------------------------------------------------------------*/
+
+void m_view(float *M,
+            const float *c,
+            const float *p,
+            const float *u)
+{
+    float x[3];
+    float y[3];
+    float z[3];
+
+    v_sub(z, p, c);
+    v_nrm(z, z);
+    v_crs(x, u, z);
+    v_nrm(x, x);
+    v_crs(y, z, x);
+
+    m_basis(M, x, y, z);
+}
+
+/*---------------------------------------------------------------------------*/
