@@ -1,4 +1,4 @@
-/*   
+/*
  * Copyright (C) 2003 Robert Kooima
  *
  * NEVERBALL is  free software; you can redistribute  it and/or modify
@@ -24,9 +24,10 @@
 
 #define fsinf(a)      ((float) sin((double) a))
 #define fcosf(a)      ((float) cos((double) a))
-#define ftanf(a)      ((float) cos((double) a))
+#define ftanf(a)      ((float) tan((double) a))
 #define fabsf(a)      ((float) fabs((double) a))
 #define fsqrtf(a)     ((float) sqrt((double) a))
+#define facosf(a)     ((float) acos((double) a))
 #define fmodf(x,y)    ((float) fmod((double) x, (double) y))
 #define fatan2f(x, y) ((float) atan2((double) x, (double) y))
 
@@ -91,7 +92,6 @@ void   m_ident(float *);
 void   m_basis(float *, const float *,
                         const float *,
                         const float *);
-
 void   m_xlt(float *, const float *);
 void   m_scl(float *, const float *);
 void   m_rot(float *, const float *, float);
@@ -100,4 +100,8 @@ void   m_mult(float *, const float *, const float *);
 void   m_pxfm(float *, const float *, const float *);
 void   m_vxfm(float *, const float *, const float *);
 
+void   m_view(float *, const float *,
+                       const float *,
+                       const float *);
+
 #endif
